@@ -4,9 +4,9 @@ vt_graph_api package exports.
 """
 
 
-from vt_graph_api.graph import VTGraph
+from vt_graph_api.graph import VTGraph, RepresentationType
 from vt_graph_api.node import Node
 from vt_graph_api.version import __version__
 
 
-__all__ = ["Node", "VTGraph", "__version__"]
+__all__ = ["Node", "VTGraph", "RepresentationType", "__version__"]
