@@ -7,7 +7,6 @@ Documentation:
   VT API: https://virustotal.github.io/vt-graph-api/
 """
 
-
 import collections
 import functools
 import json
@@ -15,6 +14,9 @@ import logging
 import os
 import threading
 
+from enum import Enum
+
+import base64
 import concurrent.futures
 import requests
 import six
@@ -25,6 +27,11 @@ import vt_graph_api.node
 import vt_graph_api.version
 
 
+class RepresentationType(str, Enum):
+  FORCE_GRAPH = "force_graph"
+  TREE = "tree"
+
+
 class VTGraph(object):
   """Python object wrapper for Virustotal Graph representation.
 
@@ -106,20 +113,20 @@ class VTGraph(object):
     # Get graph data.
     graph_data_url = (
         "https://www.virustotal.com/api/v3/graphs/{graph_id}"
-        .format(graph_id=graph_id))
+          .format(graph_id=graph_id))
     graph_data_response = requests.get(graph_data_url, headers=headers)
     if graph_data_response.status_code != 200:
       raise vt_graph_api.errors.LoadError(
           ("Error to find graph with id: {graph_id}. Response code: " +
            "{status_code}.").format(
-               graph_id=graph_id, status_code=graph_data_response.status_code))
+              graph_id=graph_id, status_code=graph_data_response.status_code))
 
     try:
       data = graph_data_response.json()
     except json.JSONDecodeError:
       raise vt_graph_api.errors.LoadError(
           "Malformed JSON response: {json_response}"
-          .format(json_response=graph_data_response.text))
+            .format(json_response=graph_data_response.text))
 
     try:
       graph_name = data["data"]["attributes"]["graph_data"]["description"]
@@ -129,7 +136,7 @@ class VTGraph(object):
     except KeyError as e:
       raise vt_graph_api.errors.InvalidJSONError(
           "Unexpected error in json structure at load_graph: {msg}."
-          .format(msg=str(e)))
+            .format(msg=str(e)))
 
     # Creates empty graph.
     graph = vt_graph_api.graph.VTGraph(
@@ -138,15 +145,22 @@ class VTGraph(object):
     # Adds users/group viewers and editors.
     graph._pull_viewers()
     graph._pull_editors()
+    # Adds group nodes from json data
+    graph._add_group_nodes_from_json_data(nodes)
+    # Adds intelligence nodes from json_data
+    graph._add_special_relationship_nodes_from_json_data(nodes)
+    # Adds intelligence links from json data
+    graph._add_special_links_from_json_data(links)
     # Adds nodes to the graph.
     graph._add_nodes_from_json_graph_data(nodes)
     # Adds links to the graph.
     graph._add_links_from_json_graph_data(links)
+
     return graph
 
   @staticmethod
   def clone_graph(graph_id, api_key, name="", private=False, user_editors=None,
-                  user_viewers=None, group_editors=None, group_viewers=None):
+      user_viewers=None, group_editors=None, group_viewers=None):
     """Clone VirusTotal Graph and make it yours according the given parameters.
 
     Args:
@@ -183,8 +197,8 @@ class VTGraph(object):
     return graph
 
   def __init__(self, api_key, name="", private=False, user_editors=None,
-               user_viewers=None, group_editors=None, group_viewers=None,
-               verbose=False):
+      user_viewers=None, group_editors=None, group_viewers=None,
+      verbose=False):
     """Creates a VT Graph Instance.
 
     Args:
@@ -217,9 +231,13 @@ class VTGraph(object):
     self.group_editors = group_editors or []
     self.group_viewers = group_viewers or []
     self.verbose = verbose
+    self.representation = RepresentationType.FORCE_GRAPH
 
     self.nodes = {}
     self.links = {}
+    self.group_nodes = {}
+    self.special_relationship_nodes = {}
+    self.special_relationship_links = {}
 
     self._id_references = {}
     self._api_calls_lock = threading.Lock()
@@ -387,20 +405,30 @@ class VTGraph(object):
     try:
       # It is necessary to clean the given links because they have relationship
       # nodes
+
+      non_special_relationship_links = (
+          link for link in json_graph_data_links if
+          link['source'] not in self.group_nodes and link[
+            'source'] not in self.special_relationship_nodes and link[
+            'target'] not in self.special_relationship_nodes and link[
+            'target'] not in self.group_nodes
+      )
+
+      relationship_links = []
+      non_relationship_links = []
+      for link in non_special_relationship_links:
+        if link["source"].startswith("relationship"):
+          relationship_links.append(link)
+        else:
+          non_relationship_links.append(link)
+
       replace_nodes = {}
-      relationship_links = (
-          link_ for link_ in json_graph_data_links
-          if link_["source"].startswith("relationship"))
       for link in relationship_links:
         if link["source"] not in replace_nodes:
           replace_nodes[link["source"]] = [link["target"]]
         else:
           replace_nodes[link["source"]].append(link["target"])
 
-      non_relationship_links = (
-          link for link in json_graph_data_links
-          if not link["source"].startswith("relationship"))
-
       for link_data in non_relationship_links:
         linked_nodes = replace_nodes.get(
             link_data["target"], [link_data["target"]])
@@ -413,6 +441,87 @@ class VTGraph(object):
           "field in the VT response."
       )
 
+  def _add_group_nodes_from_json_data(self, json_graph_data_nodes):
+    """Add all the group nodes from the given data.
+
+    json_graph_data_nodes are the responses from querying VT.
+
+    Args:
+      json_graph_data_nodes ([dict]): list of node's data with the following
+      structure => {
+          "entity_attributes": "",
+          "entity_id": "",
+          "index": "",
+          "type": "",
+          "x": "",
+          "y": ""
+      }
+    """
+
+    self.group_nodes = {node['entity_id']: node for node in
+                        json_graph_data_nodes
+                        if node.get("type") == "relationship" and node.get(
+            "entity_attributes", {}).get("relationship_type") == "group"}
+
+  def _is_special_relationship_node(self, node):
+    """Checks if a node is a special relationship node.
+
+    Args:
+      node: node to be checked.
+
+    Returns: boolean
+
+    """
+    is_relationship_node = node.get("type") == "relationship"
+    entity_attributes = node.get("entity_attributes", {})
+    relationship_type = entity_attributes.get("relationship_type")
+    is_commonalities = entity_attributes.get("commonalities") is not None
+
+    return is_relationship_node and (
+        relationship_type and relationship_type != "group") or is_commonalities
+
+  def _add_special_relationship_nodes_from_json_data(self,
+      json_graph_data_nodes):
+    """Add all the special relationship nodes from the given data.
+
+    json_graph_data_nodes are the responses from querying VT.
+
+    Args:
+      json_graph_data_nodes ([dict]): list of node's data with the following
+      structure => {
+          "entity_attributes": "",
+          "entity_id": "",
+          "index": "",
+          "type": "",
+          "x": "",
+          "y": ""
+      }
+    """
+    self.special_relationship_nodes = dict([
+        (node['entity_id'], node) for node in json_graph_data_nodes if
+        self._is_special_relationship_node(node)
+    ])
+
+  def _add_special_links_from_json_data(self, json_graph_data_links):
+    """Add all the special relationship links from the given data.
+    json_graph_data_nodes are the responses from querying VT.
+
+    Args:
+      json_graph_data_nodes ([dict]): list of node's data with the following
+      structure => {
+          "entity_attributes": "",
+          "entity_id": "",
+          "index": "",
+          "type": "",
+          "x": "",
+          "y": ""
+      }
+    """
+    self.special_relationship_links = [
+        link for link in json_graph_data_links if
+        link['source'] in self.special_relationship_nodes or
+        link['target'] in self.special_relationship_nodes]
+
   def _pull_viewers(self):
     """Pull graph's users and groups viewers from VT API.
 
@@ -447,7 +556,7 @@ class VTGraph(object):
     except KeyError as e:
       raise vt_graph_api.errors.InvalidJSONError(
           "Unexpected error in json structure at get_graph_viewers: {msg}"
-          .format(msg=str(e)))
+            .format(msg=str(e)))
     self.user_viewers.extend(user_viewers)
     self.group_viewers.extend(group_viewers)
 
@@ -514,7 +623,7 @@ class VTGraph(object):
     except KeyError as e:
       raise vt_graph_api.errors.InvalidJSONError(
           "Unexpected error in json structure at get_graph_editors: {msg}"
-          .format(msg=str(e)))
+            .format(msg=str(e)))
 
     self.user_editors.extend(user_editors)
     self.group_editors.extend(group_editors)
@@ -570,10 +679,10 @@ class VTGraph(object):
     if response.status_code != 200:
       self._log(
           "Saving graph error: {status_code} status code."
-          .format(status_code=response.status_code))
+            .format(status_code=response.status_code))
       raise vt_graph_api.errors.SaveGraphError(
           "Saving graph error: {status_code} status code."
-          .format(status_code=response.status_code)
+            .format(status_code=response.status_code)
       )
 
     data = response.json()
@@ -599,7 +708,7 @@ class VTGraph(object):
     if response.status_code != 200:
       self._log(
           "Request to '{url}' with '{status_code}' status code"
-          .format(url=url, status_code=response.status_code)
+            .format(url=url, status_code=response.status_code)
       )
       return
 
@@ -630,7 +739,7 @@ class VTGraph(object):
         # us the common expansions
         shared_expansions = (
             set(node.children)
-            .intersection(set(node_.children)))
+              .intersection(set(node_.children)))
         # Two nodes could be minimized if they have the same children in the
         # same expansion and they have at least one child.
         for expansion in shared_expansions:
@@ -789,7 +898,7 @@ class VTGraph(object):
     return valid_node_id
 
   def _query_expansion_nodes(self, node, expansion,
-                             max_nodes_per_relationship, cursor, max_retries):
+      max_nodes_per_relationship, cursor, max_retries):
     """Get expansion nodes JSON data by querying VirusTotal API.
 
     Args:
@@ -826,7 +935,7 @@ class VTGraph(object):
       try:
         self._log(
             "Expanding node {node_id} with expansion {expansion}"
-            .format(node_id=node.node_id, expansion=expansion))
+              .format(node_id=node.node_id, expansion=expansion))
         self._increment_api_counter()
         response = requests.get(
             url, headers=self._get_headers(), timeout=self.REQUEST_TIMEOUT)
@@ -840,9 +949,9 @@ class VTGraph(object):
     return data
 
   def _get_expansion_nodes(self, node, expansion,
-                           max_nodes_per_relationship=1000, cursor=None,
-                           max_retries=3, expansion_nodes=None,
-                           consumed_quotas=0):
+      max_nodes_per_relationship=1000, cursor=None,
+      max_retries=3, expansion_nodes=None,
+      consumed_quotas=0):
     """Returns the nodes to be attached to the given node with the given expansion.
 
     Args:
@@ -919,7 +1028,7 @@ class VTGraph(object):
     return expansion_nodes, consumed_quotas
 
   def _parallel_expansion(self, target_nodes, solution_paths, visited_nodes,
-                          max_api_quotas, lock, max_depth, node, params):
+      max_api_quotas, lock, max_depth, node, params):
     """Parallelize the node expansion synchronizing the api quotas consumed.
 
     Args:
@@ -1002,7 +1111,7 @@ class VTGraph(object):
     return expansion_nodes
 
   def _search_connection(self, source_node, target_nodes,
-                         max_api_quotas, max_depth, max_qps):
+      max_api_quotas, max_depth, max_qps):
     """Search connection between the node source and all of the target_nodes.
 
                           source_node
@@ -1074,8 +1183,8 @@ class VTGraph(object):
     return paths
 
   def _resolve_relations(self, source_node, target_nodes,
-                         max_api_quotas, max_depth, max_qps,
-                         fetch_info_collected_nodes):
+      max_api_quotas, max_depth, max_qps,
+      fetch_info_collected_nodes):
     """Try to connect the source_node with all of the nodes in target_nodes.
 
     Args:
@@ -1121,8 +1230,8 @@ class VTGraph(object):
     return has_link
 
   def add_node(self, node_id, node_type, fetch_information=True,
-               fetch_vt_enterprise=True, label="", node_attributes=None,
-               x=0, y=0):
+      fetch_vt_enterprise=True, label="", node_attributes=None,
+      x=0, y=0):
     """Adds a node with id `node_id` of `node_type` type to the graph.
 
     Args:
@@ -1174,7 +1283,7 @@ class VTGraph(object):
     return node
 
   def add_nodes(self, node_list, fetch_information=True,
-                fetch_vt_enterprise=True):
+      fetch_vt_enterprise=True):
     """Adds the node_list to the graph concurrently.
 
     Args:
@@ -1257,7 +1366,7 @@ class VTGraph(object):
     if source_node == target_node:
       raise vt_graph_api.errors.SameNodeError(
           "It is no possible to add links between the same node; id: {node_id}."
-          .format(node_id=source_node))
+            .format(node_id=source_node))
 
     source_node = self._get_node_id(source_node)
     target_node = self._get_node_id(target_node)
@@ -1265,11 +1374,11 @@ class VTGraph(object):
     if source_node not in self.nodes:
       raise vt_graph_api.errors.NodeNotFoundError(
           "Node '{node_id}' not found in nodes."
-          .format(node_id=source_node))
+            .format(node_id=source_node))
     if target_node not in self.nodes:
       raise vt_graph_api.errors.NodeNotFoundError(
           "Node '{node_id}' not found in nodes."
-          .format(node_id=target_node))
+            .format(node_id=target_node))
     if connection_type not in self.nodes[source_node].expansions_available:
       self._log("Expansion `{expansion_type}` is not standard expansion type",
                 logging.WARNING)
@@ -1279,8 +1388,8 @@ class VTGraph(object):
     self.nodes[source_node].add_child(target_node, connection_type)
 
   def add_links_if_match(self, source_node, target_node,
-                         max_api_quotas=100000, max_depth=3, max_qps=1000,
-                         fetch_info_collected_nodes=True):
+      max_api_quotas=100000, max_depth=3, max_qps=1000,
+      fetch_info_collected_nodes=True):
     """Try to find a relationship between the source_node and the target_node.
 
     Adds the needed links between the source_node and the target_node if
@@ -1314,7 +1423,7 @@ class VTGraph(object):
     if source_node == target_node:
       raise vt_graph_api.errors.SameNodeError(
           "It is no possible to add links between the same node; id: {node_id}."
-          .format(node_id=source_node))
+            .format(node_id=source_node))
 
     quotas_before_get_id = self.get_api_calls()
     source_node = self._get_node_id(source_node)
@@ -1341,8 +1450,8 @@ class VTGraph(object):
         max_depth, max_qps, fetch_info_collected_nodes)
 
   def connect_with_graph(self, source_node, max_api_quotas=100000,
-                         max_depth=3, max_qps=1000,
-                         fetch_info_collected_nodes=True):
+      max_depth=3, max_qps=1000,
+      fetch_info_collected_nodes=True):
     """Try to connect the source_node with the current graph nodes.
 
     Args:
@@ -1414,18 +1523,18 @@ class VTGraph(object):
     if source_node not in self.nodes:
       raise vt_graph_api.errors.NodeNotFoundError(
           "Node '{node_id}' not found in nodes."
-          .format(node_id=source_node))
+            .format(node_id=source_node))
     if target_node not in self.nodes:
       raise vt_graph_api.errors.NodeNotFoundError(
           "Node '{node_id}' not found in nodes."
-          .format(node_id=target_node))
+            .format(node_id=target_node))
 
     if (source_node, target_node, connection_type) not in self.links:
       raise vt_graph_api.errors.LinkNotFoundError(
           ("Link between {source} and {target} with {connection_type} does " +
            "not exists.").format(
-               source=source_node, target=target_node,
-               connection_type=connection_type))
+              source=source_node, target=target_node,
+              connection_type=connection_type))
     del self.links[(source_node, target_node, connection_type)]
     self.nodes[source_node].delete_child(target_node, connection_type)
 
@@ -1443,7 +1552,7 @@ class VTGraph(object):
     if node_id not in self.nodes:
       raise vt_graph_api.errors.NodeNotFoundError(
           "Node '{node_id}' not found in nodes."
-          .format(node_id=node_id))
+            .format(node_id=node_id))
 
     links_to_be_deleted = [
         link for link in six.iterkeys(self.links)
@@ -1479,13 +1588,13 @@ class VTGraph(object):
     if node_id not in six.iterkeys(self.nodes):
       raise vt_graph_api.errors.NodeNotFoundError(
           "Node '{node_id}' not found in nodes."
-          .format(node_id=node_id))
+            .format(node_id=node_id))
     node = self.nodes[node_id]
 
     if expansion not in node.expansions_available:
       raise vt_graph_api.errors.NodeNotSupportedExpansionError(
           "Node {node_id} cannot be expanded with {expansion} expansion."
-          .format(node_id=node_id, expansion=expansion))
+            .format(node_id=node_id, expansion=expansion))
 
     expansion_nodes, _ = self._get_expansion_nodes(
         node, expansion, max_nodes_per_relationship)
@@ -1543,7 +1652,7 @@ class VTGraph(object):
     return expansion_nodes
 
   def expand_n_level(self, level=1, max_nodes_per_relationship=40,
-                     max_nodes=10000):
+      max_nodes=10000):
     """Expands all the nodes in the graph `level` levels.
 
     For example:
@@ -1611,6 +1720,7 @@ class VTGraph(object):
                 "private": self.private,
                 "nodes": [],
                 "links": [],
+                "representation": self.representation
             },
             "type": "graph",
         },
@@ -1657,6 +1767,24 @@ class VTGraph(object):
       self._add_node_to_output(output, node_id)
       added.add(node_id)
 
+    special_relationship_nodes = self._get_special_relationship_nodes(
+        len(added))
+    for node in special_relationship_nodes:
+      output["data"]["attributes"]["nodes"].append(node)
+      added.add(node['entity_id'])
+
+    special_relationship_links = self._get_special_relationship_links(added)
+    output["data"]["attributes"]["links"] += special_relationship_links
+
+    group_nodes = self._get_groups_nodes(added)
+    for node in group_nodes:
+      output["data"]["attributes"]["nodes"].append(node)
+      added.add(node['entity_id'])
+
+    final_links = output["data"]["attributes"]["links"]
+    group_links = self._get_groups_links(final_links)
+    output["data"]["attributes"]["links"] += group_links
+
     self._push_graph_to_vt(output)
     self._push_editors()
     self._push_viewers()
@@ -1704,7 +1832,7 @@ class VTGraph(object):
         "{graph_id}\" width=\"800\" height=\"600\"></iframe>"
         .format(graph_id=self.graph_id))
 
-  def download_screenshot(self, path = "."):
+  def download_screenshot(self, path="."):
     """Downloads a screenshot of the graph.
 
     Args:
@@ -1732,7 +1860,7 @@ class VTGraph(object):
       r.raw.decode_content = True
       filename = "{graph_id}.jpg".format(graph_id=self.graph_id)
       file_path = os.path.join(path, filename)
-      with open(file_path,'wb') as f:
+      with open(file_path, 'wb') as f:
         shutil.copyfileobj(r.raw, f)
     else:
       raise vt_graph_api.errors.DownloadScreenshotError(
@@ -1747,6 +1875,98 @@ class VTGraph(object):
         for node in self.nodes.values() if node.node_type == node_type
     ]}
 
+  def _get_groups_nodes(self, final_nodes):
+    """This method must be called only just before saving the graph.
+    It returns all the loaded and generated group nodes. It checks that
+    the nodes that belong to the group have been added to the graph, and deletes
+    the ids that have not finally been added.
+
+    In case a Group Node doesn't have any belonging node in the Graph, it is not
+    returned.
+
+    Args:
+      final_nodes: Set with the ids of all the added nodes to the Graph.
+
+    Returns: Array of group nodes.
+
+    """
+    group_nodes = []
+    next_index = len(final_nodes)
+    # Delete nodes belonging to group nodes that have not been added to the Graph.
+    # Inject Node index.
+    for idx, group_node in enumerate(self.group_nodes.values()):
+      belonging_nodes = group_node['entity_attributes']['grouped_node_ids']
+      belonging_nodes = list(
+          set(node for node in belonging_nodes if node in final_nodes))
+      if belonging_nodes:
+        group_node['entity_attributes']['grouped_node_ids'] = belonging_nodes
+        group_node['index'] = next_index
+        next_index += 1
+        group_nodes.append(group_node)
+    return group_nodes
+
+  def _get_groups_links(self, final_links):
+    """Generates all the links needed to represent groups in VTGraph.
+
+    Args:
+      final_links: list with all the links added in the Graph.
+
+    Returns: List with the links needed to represent groups in VTGraph.
+
+    """
+    map_node_group = {}
+
+    # Generate a link for each pair node_in_group -> group_node
+    for group_node in self.group_nodes.values():
+      for node_in_group in group_node['entity_attributes']['grouped_node_ids']:
+        map_node_group[node_in_group] = group_node['entity_id']
+
+    group_links = [{'source': node, 'target': group, 'connection_type': 'group'}
+                   for node, group in map_node_group.items()]
+
+    # Generate a link for each link where source or target is a node
+    # inside a group.
+    for link in final_links:
+      if link['source'] in map_node_group:
+        new_link = link.copy()
+        new_link['source'] = map_node_group[link['source']]
+        group_links.append(new_link)
+      elif link['target'] in map_node_group:
+        new_link = link.copy()
+        new_link['target'] = map_node_group[link['target']]
+        group_links.append(new_link)
+    return group_links
+
+  def _get_special_relationship_nodes(self, next_index):
+    """
+    Returns a list with the special relationship nodes, and injects nodes
+    index. Only used for saving the Graph.
+    Args:
+      next_index: Next index to be injected.
+
+    Returns:
+
+    """
+    special_relationship_nodes = []
+    for idx, node in self.special_relationship_nodes.values():
+      node['index'] = next_index + idx
+      special_relationship_nodes.append(node)
+    return special_relationship_nodes
+
+  def _get_special_relationship_links(self, final_nodes):
+    """Returns links needed to represent special relationship nodes.
+    It checks if the link connections exist or not. Only used for saving the
+    Graph.
+
+    Args:
+      final_nodes: Set with all the nodes added
+
+    Returns: list of links related to special_relationship_nodes.
+
+    """
+    return [link for link in self.special_relationship_links if
+            link['source'] in final_nodes and link['target'] in final_nodes]
+
   def create_collection(self, name=None, description=None):
     """Creates a VT Collection taking entities from current Graph.
 
@@ -1765,13 +1985,13 @@ class VTGraph(object):
     data = {
         "type": "collection",
         "attributes": {
-          "name": name if name else "Collection created from VT Graph API",
+            "name": name if name else "Collection created from VT Graph API",
         },
         "relationships": {
-          "files": self._get_nodes_by_type('file'),
-          "domains": self._get_nodes_by_type('domain'),
-          "urls": self._get_nodes_by_type('url'),
-          "ip_addresses": self._get_nodes_by_type('ip_address')
+            "files": self._get_nodes_by_type('file'),
+            "domains": self._get_nodes_by_type('domain'),
+            "urls": self._get_nodes_by_type('url'),
+            "ip_addresses": self._get_nodes_by_type('ip_address')
         }
     }
 
@@ -1786,7 +2006,7 @@ class VTGraph(object):
     response = requests.post(
         url, headers=self._get_headers(), json={"data": data})
 
-    if(response.status_code != 200):
+    if (response.status_code != 200):
       print(response.json())
       raise vt_graph_api.errors.CreateCollectionError()
 
@@ -1796,9 +2016,64 @@ class VTGraph(object):
         collection_id=collection_id
     )
 
+  def _generate_group_node_id(self, nodes_id):
+    return ",".join(nodes_id).replace("=", "")
 
+  def create_group(self, node_ids, group_name):
+    """
 
+    Args:
+      node_ids:
+      group_name:
 
+    Returns:
 
+    Raises: CreateGroupError if:
+      - Group doesn't contain any node.
+      - Group contains a node that is already in another group.
+      - Group contains a node that is not in the Graph.
 
+    """
+    group_node_id = self._generate_group_node_id(node_ids)
+    node_ids_set = set(node_ids)
 
+    # Check if the user has provided node ids
+    if not node_ids:
+      raise vt_graph_api.errors.CreateGroupError(
+          "A group must contain at least one node.")
+
+    # Check if all the nodes exists.
+    for node_id in node_ids_set:
+      if node_id not in self.nodes:
+        raise vt_graph_api.errors.CreateGroupError(
+            "Node {node_id} is not in the Graph.".format(
+                node_id=node_id))
+
+    # Check if nodes are already in a group.
+    for group_node in self.group_nodes.values():
+      for node_id in group_node["entity_attributes"]["grouped_node_ids"]:
+        if node_id in node_ids_set:
+          raise vt_graph_api.errors.CreateGroupError(
+              "Node {node_id} is already in a group.".format(
+                  node_id=node_id))
+
+    relationship_node = {
+        "entity_id": "relationships_group_{group_id}".format(
+            group_id=group_node_id),
+        "type": 'relationship',
+        "entity_attributes": {
+            "grouped_node_ids": node_ids,
+            "relationship_type": "group"
+        },
+        "text": group_name
+    }
+
+    self.group_nodes[group_node_id] = relationship_node
+
+  def set_representation(self, representation):
+    """Sets Graph representation.
+
+    Args:
+      representation: Graph representation. See :py:class::`RepresentationType`.
+    """
+    self.representation = representation
